@@ -1,6 +1,6 @@
 ;;; (tests test-parser) --- Guile JSON implementation.
 
-;; Copyright (C) 2018 Aleix Conchillo Flaque <aconchillo@gmail.com>
+;; Copyright (C) 2018-2020 Aleix Conchillo Flaque <aconchillo@gmail.com>
 ;;
 ;; This file is part of guile-json.
 ;;
@@ -36,13 +36,22 @@
 (test-equal 1234 (json-string->scm "1234"))
 (test-equal -1234 (json-string->scm "-1234"))
 (test-equal -54.897 (json-string->scm "-54.897"))
+(test-equal 0 (json-string->scm "0"))
+(test-equal 0.12 (json-string->scm "0.12"))
+(test-equal 0.0 (json-string->scm "0e0"))
 (test-equal 1000.0 (json-string->scm "1e3"))
 (test-equal 0.001 (json-string->scm "1E-3"))
+(test-error #t (json-string->scm "00"))
+(test-error #t (json-string->scm "12a34"))
+(test-error #t (json-string->scm "1k-3"))
+(test-error #t (json-string->scm "1E-3,"))
+(test-error #t (json-string->scm "1E-3p"))
 
 ;; Strings
 (test-equal "hello guile!" (json-string->scm "\"hello guile!\""))
 (test-equal "你好 guile!" (json-string->scm "\"你好 guile!\""))
 (test-equal "你好 guile!" (json-string->scm "\"\\u4f60\\u597d guile!\""))
+(test-equal "hello quoted \"guile\"!" (json-string->scm "\"hello quoted \\\"guile\\\"!\""))
 
 ;; Boolean
 (test-equal #t (json-string->scm "true"))
@@ -54,17 +63,19 @@
 ;; Arrays
 (test-equal #() (json-string->scm "[]"))
 (test-equal #(1 2 3 4) (json-string->scm "[1,2,3,4]"))
-(test-equal #(1 2 3 4) (json-string->scm "[   1,2, 3,4  ]"))
+(test-equal #(1 2 3 4) (json-string->scm "   [   1  , 2 , 3,4  ]    "))
 (test-equal #(1 2 #(3 4) #(5 6 #(7 8))) (json-string->scm "[1,2,[3,4],[5,6,[7,8]]]" ))
 (test-equal #(1 "two" 3 "four") (json-string->scm "[1,\"two\",3,\"four\"]"))
 (test-error #t (json-string->scm "[1,2,,,5]"))
 
 ;; Objects
+(test-equal '() (json-string->scm "{}"))
 (test-equal '(("foo" . "bar")) (json-string->scm "{\"foo\":\"bar\"}"))
-(test-equal '(("foo" . "bar")) (json-string->scm "{\"foo\":\"bar\"}"))
-(test-equal '(("foo" . #(1 2 3))) (json-string->scm "{\"foo\":[1,2,3]}"))
-(test-equal '(("foo" . (("bar" . #(1 2 3))))) (json-string->scm "{\"foo\":{\"bar\":[1,2,3]}}"))
+(test-equal '(("foo" . #(1 2 3))) (json-string->scm "{\"foo\"   :   [1,2,3]}"))
+(test-equal '(("foo" . (("bar" . #(1 2 3))))) (json-string->scm "{\"foo\"   :{\"bar\":  [1,2,3]}}"))
 (test-equal '(("foo" . #(1 (("two" . "three"))))) (json-string->scm "{\"foo\":[1,{\"two\":\"three\"}]}"))
+(test-error #t (json-string->scm "{\"foo\":\"bar\",}"))
+(test-error #t (json-string->scm "{,}"))
 
 ;; Since the following JSON object contains more than one key-value pair, we
 ;; can't use "test-equal" directly since the output could be unordered.
@@ -73,6 +84,13 @@
 (test-equal "An author" (assoc-ref book "author"))
 (test-equal 29.99 (assoc-ref book "price"))
 
+;; Some extra errors
+(test-error #t (json-string->scm "{"))
+(test-error #t (json-string->scm "]"))
+(test-error #t (json-string->scm "we are missing the double-quotes"))
+(test-error #t (json-string->scm "[1,2,3] extra"))
+(test-error #t (json-string->scm "{} extra"))
+
 (exit (if (test-end "test-parser") 0 1))
 
 ;;; (tests test-parser) ends here
